#include <string>
#include <iostream>
#include <vector>
#include <SDL/SDL.h>
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "GameEngine.h"
#include <iostream>
#include <fstream>

/* ---------- GameEngine Methods ---------- */

GameEngine::GameEngine()
{
	//Screen attributes 
	SCREEN_WIDTH = 640;
	SCREEN_HEIGHT = 480;
	SCREEN_BPP = 32;
	
	//The surfaces 
	background = NULL; 
	screen = NULL;
	
	//Initialize all SDL subsystems 
	if( SDL_Init( SDL_INIT_EVERYTHING ) == -1 ){ 
		SDL_Quit();
	} 
	
	//Set up the screen 
	screen = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_SWSURFACE );
	
	//If there was an error in setting up the screen 
	if( screen == NULL ) { 
		SDL_Quit();
	}
	
	//Initialize SDL_ttf 
	if( TTF_Init() == -1 ) { 
		SDL_Quit();
	}
	
	//Set the window caption 
	SDL_WM_SetCaption( "Arcade Game", NULL );
	
	//set fps to 20 frames/sec
	fps = 20;
	
	entername = false;
	
	//Initialize the string
   	 str = "";

	//Initialize name for High Score and text for information-lines
	name = new Text();
	text = new Text();
	name->SetPos(100, 350);
	text->SetPos(300, 350);
}

GameEngine::~GameEngine()
{
	if (state != NULL) {
		delete state;
		state = NULL;
	}
}

void GameEngine::SetState(GameState* newstate)
{
	delete state;
	state = newstate;
	state->SetEngine(this);
}

SDL_Surface* GameEngine::GetScreen()
{
	return screen;
}

void GameEngine::Loop()
{
	//The event structure that will be used 
	SDL_Event event;
	
	//Keep time to next frame (fps)
	Uint32 nextFrame;
	
	//Make sure the program waits for a quit 
	quit = false;
	
	//While the user hasn't quit 
	while( quit == false ) 
	{
		//Start the frame timer 
		nextFrame = SDL_GetTicks() + fps;
		
		//While there's an event to handle
		if( SDL_PollEvent( &event ) ) {
			//If a key was pressed 
			if( event.type == SDL_KEYDOWN ) {
				
				//If the user hasn't entered their name yet
            			if( entername == true && nameentered == false )	{
            			//Keep a copy of the current version of the string
        			std::string temp = str;

					//If the string less than maximum size
					if( str.length() <= 16 ) {
						//If the key is a space
						if( event.key.keysym.unicode == (Uint16)' ' ) {
     			           			//Append the character
     			           			str += (char)event.key.keysym.unicode;    
       				     		}
            					//If the key is a number
            					else if( ( event.key.keysym.unicode >= (Uint16)'0' ) && ( event.key.keysym.unicode <= (Uint16)'9' ) ) {
                					//Append the character
                					str += (char)event.key.keysym.unicode;
            					}
            					//If the key is a uppercase letter
            					else if( ( event.key.keysym.unicode >= (Uint16)'A' ) && ( event.key.keysym.unicode <= (Uint16)'Z' ) ) {
                					//Append the character
               						str += (char)event.key.keysym.unicode;
            					}
            					//If the key is a lowercase letter
            					else if( ( event.key.keysym.unicode >= (Uint16)'a' ) && ( event.key.keysym.unicode <= (Uint16)'z' ) ) {
                					//Append the character
                					str += (char)event.key.keysym.unicode;
            					}
            					else if (event.key.keysym.sym == SDLK_RETURN) {
            						//Change the flag
                    					nameentered = true;
            					}
        				}
    					//If backspace was pressed and the string isn't blank
        				if( ( event.key.keysym.sym == SDLK_BACKSPACE ) && ( str.length() != 0 ) ) {
            					//Remove a character from the end
            					str.erase( str.length() - 1 );
        				}
        			
        				//If the string was changed
        				if( str != temp ) {
            					//Render a new text surface
            					name->Render(str.c_str());
		      				if (nameentered == false) {
	        					text->Render("Push Enter to confirm name");
						}
        				}
            			}
            		else {
            			switch( event.key.keysym.sym ) 
					{ 
						case SDLK_ESCAPE: quit = true; break;
						case SDLK_RETURN: state->KeyPressed(3); break;
						case SDLK_SPACE: state->KeyPressed(4); break;
					} 
				}
			}
			
			//If the user has Xed out the window 
			else if( event.type == SDL_QUIT ) { 
				//Quit the program 
				quit = true; 
			}
		}
		
		//Get the keystates 
		Uint8 *keystates = SDL_GetKeyState( NULL );
		
		//If left is pressed 
		if( keystates[ SDLK_LEFT ] ) { 
			state->KeyPressed(1);
		} 
		//If right is pressed 
		if( keystates[ SDLK_RIGHT ] ) {
			state->KeyPressed(2); 
		}

		//Move non-player sprites (if any)
		state->MoveSprites();
		
		//Redraw screen
		state->Draw(screen);
		
		if (entername == true && nameentered == false) {
			name->Draw(screen);
			text->Draw(screen);
		}
		else if (entername == true && nameentered == true) {
			text->Render("Push Enter to add your score");
			text->Draw(screen);
		}
		
		SDL_Flip(screen);
		
		//If we want to cap the frame rate 
		if( SDL_GetTicks() < nextFrame ) { 
			//Sleep the remaining frame time 
			SDL_Delay( nextFrame - SDL_GetTicks() ); 
		}
	}
}

SDL_Surface* GameEngine::load_image( std::string filename )
{
	//The image that's loaded 
	SDL_Surface* loadedImage = NULL; 
	
	//The optimized image that will be used 
	SDL_Surface* optimizedImage = NULL; 
	
	//Load the image 
	loadedImage = IMG_Load( filename.c_str() ); 
	
	//If the image loaded 
	if( loadedImage != NULL ) { 
		//Create an optimized image 
		optimizedImage = SDL_DisplayFormat( loadedImage );
		
		//If the image was optimized just fine 
		if( optimizedImage != NULL ) { 
			//Map the color key 
			Uint32 colorkey = SDL_MapRGB( optimizedImage->format, 255, 128, 0 );
			
			//Set all pixels of color R 0, G 0xFF, B 0xFF to be transparent 
			SDL_SetColorKey( optimizedImage, SDL_SRCCOLORKEY, colorkey ); 
		}
		
		//Free the old image 
		SDL_FreeSurface( loadedImage ); 
	} 
	
	//Return the optimized image 
	return optimizedImage; 
}

void GameEngine::SetEntername()
{
	if (entername == false) {
		entername = true;
	} else {
		entername = false;
	}
}

void GameEngine::SetNameentered()
{
	if (nameentered == false) {
		nameentered = true;
	} else {
		nameentered = false;
	}
}

void GameEngine::SetName()
{
	state->SetName(str);
}


/* ---------- GameState Methods ---------- */

GameState::GameState()
{
	this->engine = NULL;
}

GameEngine* GameState::SetEngine(GameEngine* newengine)
{
	engine = newengine;
}

GameEngine* GameState::GetEngine()
{
	return engine;
}

void GameState::MoveSprites()
{
}

void GameState::Draw(SDL_Surface* surface)
{
}

void GameState::KeyPressed(int key)
{
}

void GameState::SetName(std::string newname)
{
	name = newname;
}

std::string GameState::GetName()
{
	return name;
}


/* ---------- Sprite Methods ---------- */

Sprite::Sprite()
{
	image = NULL;
}

Sprite::~Sprite()
{
	if (image != NULL) {
		SDL_FreeSurface(image);
		image = NULL;
	}
}

int Sprite::GetPosX()
{
	return pos_x;
}

int Sprite::GetPosY()
{
	return pos_y;
}

void Sprite::SetPos(int posX, int posY)
{
	pos_x = posX;
	pos_y = posY;
}

void Sprite::SetImage(SDL_Surface* new_image)
{
	if (image != NULL) {
		SDL_FreeSurface(image);
		image = NULL;
	}
	image = new_image;
}

SDL_Surface* Sprite::GetImage()
{
	return image;
}

void Sprite::MoveLeft()
{	
}

void Sprite::MoveRight()
{	
}

void Sprite::MoveUp()
{
}

void Sprite::MoveDown()
{
}

bool Sprite::CollisionCheck(int posX, int posY, int sizeX, int sizeY)
{
	if (pos_y > posY && pos_y < (posY+sizeY) && (pos_x+5) > posX && pos_x < (posX+sizeX)) {
		return true;
	} else {
		return false;
	}
}

void Sprite::Draw(SDL_Surface* target)
{
	SDL_Rect offset;
	offset.x = pos_x;
	offset.y = pos_y;

	SDL_BlitSurface(GetImage(), NULL, target, &offset);
}

void Sprite::SetDel()
{
}

bool Sprite::GetDel()
{
}


/* ---------- Text Methods ---------- */

Text::Text()
{
	//Open the font 
	font = TTF_OpenFont( "arial.ttf", 22 );
	text = NULL;

    	//Enable Unicode
    	SDL_EnableUNICODE( SDL_ENABLE );
}

Text::~Text()
{
	SDL_FreeSurface( text );
	TTF_CloseFont( font );
}

void Text::Render(const char* message)
{
	//Render the text
	SDL_Color textColor = { 255, 0, 255 };
	text = TTF_RenderText_Blended( font, message, textColor );
}

void Text::Draw(SDL_Surface* target)
{
	SDL_Rect offset;
	offset.x = pos_x;
	offset.y = pos_y;

	SDL_BlitSurface(text, NULL, target, &offset);
}

void Text::SetPos(int posX, int posY)
{
	pos_x = posX;
	pos_y = posY;
}

