#ifndef GAMEENGINE_H
#define GAMEENGINE_H

#include <SDL/SDL.h>
#include <string>

class GameState;
class Text;

class GameEngine
{
	private:
		int SCREEN_WIDTH;
		int SCREEN_HEIGHT;
		int SCREEN_BPP;
		SDL_Surface* screen;
		SDL_Surface* background;
		GameState* state;
		bool quit;
		int fps;
		bool entername;
		bool nameentered;
		Text* name;
		Text* text;
		std::string str;
	public:
		GameEngine();
		~GameEngine();
		void SetState(GameState* state);
		SDL_Surface* GetScreen();
		void Loop();
		static SDL_Surface* load_image( std::string filename );
		void SetEntername();
		void SetNameentered();
		void SetName();
};

class GameState
{
	private:
		GameEngine* engine;
		SDL_Surface* Screen;
		std::string name;
	public:
		GameState();
		GameEngine* SetEngine(GameEngine* engine);
		GameEngine* GetEngine();
		void SetName(std::string newname);
		std::string GetName(); 
		virtual void MoveSprites();
		virtual void Draw(SDL_Surface* surface);
		virtual void KeyPressed(int key);
};

class Sprite
{
	private:
		SDL_Surface* image;
		int pos_x;
		int pos_y;
	public:
		Sprite();
		virtual ~Sprite();
		SDL_Surface* GetImage();
		int GetPosX();
		int GetPosY();
		void SetPos(int pos_x, int pos_y);
		void SetImage(SDL_Surface* new_image);
		virtual void MoveLeft();
		virtual void MoveRight();
		virtual void MoveUp();
		virtual void MoveDown();
		virtual bool CollisionCheck(int posX, int posY, int sizeX, int sizeY);
		virtual void Draw(SDL_Surface* target);
		virtual void SetDel();
		virtual bool GetDel();
};

class Text
{
	private:
		TTF_Font* font;
		SDL_Surface* text;
		int pos_x;
		int pos_y;
	public:
		Text();
		~Text();
		void Render(const char* message);
		void Draw(SDL_Surface* target);
		void SetPos(int posX, int posY);
};

#endif

